'''
Created on 2014.03.27.

Sample program for Edaq530 measurements with a simple GUI.
Diplays the data measured by the device in a window.
The GUI is created using the TkInter package.
For help run the program with '-h' argument.

'''

import edaqcomm
import Tkinter
from Tkinter import *
import time
import thread
import threading
import sys
import argparse


DEFAULT_DISPLAY_REFRESH_RATE = 5.0

INVALID_CHANNEl_TPYE_MSG = "Invalid channeltype argument %s.\nValid arguments are: voltage, resistance, temp_kelvin, temp_celsius, inamp, photo.\nThe program now exit..."



class TkEdaqMeas:

	'''
    	Sets the connection to the device
    	'''
    	def setConnection(self):
    		self.argsToChType(self.args.chType)
    		self.comm = edaqcomm.Edaq530Comm(self.args.com)
		self.comm.connect()
		print(self.comm.getID())
		
		self.comm.setInputs(self.chType,
			edaqcomm.ChannelType.voltage, edaqcomm.ChannelType.voltage)
			
	'''
	Sets attributes related to channel type, uses a string
	'''
	def argsToChType(self, arg):
		if arg == "temp_celsius":
			self.chType = edaqcomm.ChannelType.resistor
			self.qType = edaqcomm.QuantityType.temp_celsius
			self.unit = "C"
		elif arg == "temp_kelvin":
			self.chType = edaqcomm.ChannelType.resistor
			self.qType = edaqcomm.QuantityType.temp_kelvin
			self.unit = "K"
		elif arg == "resistance":
			self.chType = edaqcomm.ChannelType.resistor
			self.qType = edaqcomm.QuantityType.resitance
			self.unit = "Ohm"
		elif arg == "voltage":
			self.chType = edaqcomm.ChannelType.voltage
			self.qType = edaqcomm.QuantityType.voltage
			self.unit = "V"
		elif arg == "inamp":
			self.chType = edaqcomm.ChannelType.inamp
			self.qType = edaqcomm.QuantityType.voltage
			self.unit = "V"
		elif arg == "photo":
			self.chType = edaqcomm.ChannelType.photo
			self.qType = edaqcomm.QuantityType.voltage
			self.unit = "V"
		else:
			print(INVALID_CHANNEl_TPYE_MSG % (arg))
			sys.exit(1)	
	
	
	'''
	Parse arguments using argparse
	'''
	def parseArgs(self):
		self.parser = argparse.ArgumentParser(description = 'One channel Edaq530\
		 measurement with a simple display.')
		 
		self.parser.add_argument('com', help='the name of the com port,\
		 for example /dev/ttyUSB0')
		 
		self.parser.add_argument('-cht', '--chtype', help='channel type, voltage\
		 by default. Possible arguments are: voltage, voltage_diff,\
		 resistance, temp_kelvin, temp_celsius, inamp, photo', 
			default='voltage', dest='chType')
		
		self.parser.add_argument('-rr','--refreshrate', help='display refresh rate,\
		 %.1f Hz by default' % DEFAULT_DISPLAY_REFRESH_RATE,
			default=DEFAULT_DISPLAY_REFRESH_RATE, type=float, dest='refreshRate')
		  
		self.args = self.parser.parse_args() 
	

	'''
	Display, runs on a thread
	'''
	def refreshDisplay(self):
		with self.dispRefreshingLock:
				refreshingDisp = self.refreshingDisp
		while refreshingDisp == True:
  			start = time.clock()
  			data = self.comm.measure()
  			data = edaqcomm.getMeasuredQuantity(data[0], self.qType)
  			self.display["text"] = "%.2f %s" % (data, self.unit)
			sleeptime = (1.0/self.args.refreshRate) - (time.clock()-start)
			with self.dispRefreshingLock:
				refreshingDisp = self.refreshingDisp
			if sleeptime > 0:
				time.sleep(sleeptime)

	'''
	Called, when the window is being closed.
	Stops the measurement, disconnects the device, then exits.
	'''
	def closeWindow(self):
		with self.dispRefreshingLock:
			self.refreshingDisp =  False

		self.comm.disconnect()
		self.root.destroy()
		

    	def __init__(self):
    		self.parseArgs()
    		self.root = Tk()
		self.root.title("Edaq measurement")
		self.root.protocol('WM_DELETE_WINDOW', self.closeWindow)
        	self.frame = Tkinter.Frame(self.root)
		self.display = Tkinter.Label(self.frame, font =("Arial", 50), bg = 				"white", justify = RIGHT, borderwidth = 20)
		self.display["text"] = "0 C"
		self.display.pack()
		self.frame.pack(fill=BOTH, expand=1)
		self.setConnection()

		self.refreshingDisp = True
		self.dispRefreshingLock = threading.Lock()
		
		thread.start_new_thread(self.refreshDisplay, ())

		self.root.mainloop()
		
TkEdaqMeas()

 		
